<?php

/**
 * @file
 * Contains \Drupal\simple_fb_connect\Controller\SimpleFbConnectController.
 */
namespace Drupal\simple_fb_connect\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Facebook\FacebookRedirectLoginHelper;

/**
 * Returns responses for Simple FB Connect module routes.
 */
class SimpleFbConnectController extends ControllerBase {

  protected $fb_manager;
  protected $user_manager;

  /**
   * Constructor
   *
   * The constructor parameters are passed from the create() method.
   */
  public function __construct($fb_manager, $user_manager) {
    $this->fb_manager = $fb_manager;
    $this->user_manager = $user_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('simple_fb_connect.fb_manager'),
      $container->get('simple_fb_connect.user_manager')
    );
  }

  /**
   * Response for path 'user/simple-fb-connect'
   *
   * Redirects the user to FB for authentication.
   *
   * @todo add destination handling: https://www.drupal.org/node/2524328
   */
  public function redirectToFb() {
    // Validate configuration
    if (!$this->fb_manager->validateConfig()) {
      drupal_set_message(t('Simple FB Connect not configured properly. Contact site administrator.'), 'error');
      return $this->redirect('user.login');
    }

    // TODO: save destination to session if it is defined
    // https://www.drupal.org/node/2524328

    // Redirect the user to FB for authentication
    $fb_login_url = $this->fb_manager->getFbLoginUrl();
    return new TrustedRedirectResponse($fb_login_url);

    // This should never be reached, user should have been redirected already
    throw new AccessDeniedHttpException();
  }

  /**
   * Response for path 'user/simple-fb-connect/return'
   *
   * Facebook returns the user here after user has authenticated in FB
   */
  public function returnFromFb() {
    // Validate configuration
    if (!$this->fb_manager->validateConfig()) {
      drupal_set_message(t('Simple FB Connect not configured properly.'), 'error');
      return $this->redirect('user.login');
    }

    // FB SDK can start FacebookSession from the page where FB returned the user
    $login_helper = new FacebookRedirectLoginHelper($this->fb_manager->getReturnUrl());
    if (!$this->fb_manager->startFbSession($login_helper)) {
      drupal_set_message(t("Facebook login failed."), 'error');
      return $this->redirect('user.login');
    }

    // Get a validated FacebookSession object
    if (!$fb_session = $this->fb_manager->getFbSession()) {
      drupal_set_message(t("Facebook login failed."), 'error');
      return $this->redirect('user.login');
    }

    // Get user's FB profile from Facebook API
    if (!$fb_profile = $this->fb_manager->getFbProfile($fb_session)) {
      drupal_set_message(t("Facebook login failed."), 'error');
      return $this->redirect('user.login');
    }

    // Get user's email from the FB profile
    if (!$email = $this->fb_manager->getEmail($fb_profile)) {
      drupal_set_message(t('Facebook login failed. This site requires permission to get your email address.'), 'error');
      return $this->redirect('user.login');
    }

    // Check if we have an existing Drupal user with the same email address and try to log in
    if ($drupal_user = $this->user_manager->getUserByEmail($email)) {
      if ($this->user_manager->loginUser($drupal_user)) {
        return $this->redirect($this->user_manager->getPostLoginRoute());
      }
      else {
        return $this->redirect('user.login');
      }
    }

    // If there was no existing user, try to create a new user
    $drupal_user = $this->user_manager->createUser(
      $fb_profile->getProperty('name'),
      $email,
      $fb_profile->getProperty('id'),
      $this->fb_manager->getFbProfilePicUrl($fb_session)
    );

    // Log the newly created user in
    if ($drupal_user) {
      if ($this->user_manager->loginUser($drupal_user)) {
        return $this->redirect($this->user_manager->getPostLoginRoute());
      }
      else {
        // New user was succesfully created but the account is pending approval
        drupal_set_message(t('You will receive an email when site administrator activates your account.'), 'warning');
        return $this->redirect('user.login');
      }
    }
    else {
      // User could not be created
      return $this->redirect('user.login');
    }

    // This should never be reached, user should have been redirected already
    throw new AccessDeniedHttpException();
  }

}
