<?php

/**
 * @file
 * Contains \Drupal\simple_fb_connect\SimpleFbConnectFbManager.
 */
namespace Drupal\simple_fb_connect;

use Facebook\FacebookSession;
use Facebook\FacebookRedirectLoginHelper;
use Facebook\FacebookRequestException;
use Facebook\FacebookRequest;
use Facebook\FacebookSDKException;
use Facebook\GraphObject;
use Drupal\Core\Url;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;


/**
 * Contains all Simple FB Connect logic that is related to Facebook
 * interaction.
 */
class SimpleFbConnectFbManager {

  protected $appId;
  protected $appSecret;
  protected $scope;
  protected $returnUrl;
  protected $maxResolution;
  protected $minResolution;
  protected $configFactory;
  protected $loggerFactory;
  protected $site_config;

  /**
   * Constructor
   */
  public function __construct(ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory) {
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
    $module_config =  $this->configFactory->get('simple_fb_connect.settings');
    $this->appId = $module_config->get('app_id');
    $this->appSecret = $module_config->get('app_secret');
    $this->scope = array('email');
    global $base_insecure_url;
    $this->returnUrl = $base_insecure_url.'/'.Url::fromRoute('simple_fb_connect.return_from_fb', array(), array('absolute' => TRUE))->getInternalPath();

    $site_config =  $this->configFactory->get('field.user.user.user_picture');
    $this->maxResolution = $site_config->get('settings.max_resolution');
    $this->minResolution = $site_config->get('settings.min_resolution');

    FacebookSession::setDefaultApplication($this->appId, $this->appSecret);
  }

  /**
   * Checks that module is configured
   *
   * @return bool
   *   True if module is configured
   *   False otherwise
   */
  public function validateConfig() {
    // Check that App ID and App Secret have been defined
    if (!$this->appId || !$this->appSecret) {
      $this->loggerFactory->get(simple_fb_connect)->error('Define App ID and App Secret on module settings.');
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Returns the Facebook login URL where user will be redirected
   *
   * @todo Add capability for other modules to modify scope
   *   https://www.drupal.org/node/2524378
   *
   * @return string
   *   Absolute Facebook login URL where user will be redirected
   */
  public function getFbLoginUrl() {
    $login_helper = new FacebookRedirectLoginHelper($this->returnUrl);
    $login_helper->disableSessionStatusCheck();

    // TODO: add capability for other modules to modify scope
    // Currently we are only using $this->scope defined in constructor
    // https://www.drupal.org/node/2524378
    $final_scope = $this->scope;
    return $login_helper->getLoginUrl($final_scope);
  }

  /**
   * Returns the URL where Facebook will return the user after FB authentication
   *
   * @return string
   *   Absolute URL where Facebook will return the user
   */
  public function getReturnUrl() {
    return $this->returnUrl;
  }

  /**
   * Starts FacebookSession after FB has returned the user back to our site
   *
   * Saves the user access token to Drupal session. This function must only be
   * called from route simple_fb_connect.return_from_fb because login helper
   * will use the URL parameters set by Facebook to start Facebook session.
   *
   * @param \Facebook\FacebookRedirectLoginHelper $login_helper
   *
   * @return bool
   *   True if we have a valid FB Session
   *   False otherwise
   */
  public function startFbSession(FacebookRedirectLoginHelper $login_helper) {
    try {
      $fb_session = $login_helper->getSessionFromRedirect();

      if ($fb_session) {
        $session = \Drupal::service('session');
        $session->set('simple_fb_connect_user_token', $fb_session->getToken());
        return TRUE;
      }
      else {
        // If the user cancels the dialog or return URL is invalid,
        // Facebook will not throw an exception but will return NULL
        $this->loggerFactory->get(simple_fb_connect)->error('Could not start FB session. User cancelled the dialog in FB or return URL was not valid.');
        return FALSE;
      }
    }
    catch (FacebookRequestException $ex) {
      $this->loggerFactory->get(simple_fb_connect)->error('Could not start FB session. FacebookRequestException: @message', array('@message' => json_encode($ex->getResponse())));
    }
    catch (\Exception $ex) {
      $this->loggerFactory->get(simple_fb_connect)->error('Could not start FB session. Exception: @message', array('@message' => ($ex->getMessage())));
    }

    return FALSE;
  }

  /**
   * Returns FacebookSession for the current user.
   *
   * Modules can use this function to get FacebookSession so that they
   * can make their own Facebook API requests. This function validates the
   * session (i.e. checks that user's access token from Drupal session variables
   * is still valid and from our app).
   *
   * @return \Facebook\FacebookSession|false
   *   FacebookSession if the token was valid
   *   False otherwise
   */
  public function getFbSession() {
    try {
      // Get user access token from Drupal session variables
      $session = \Drupal::service('session');
      $user_token = $session->get('simple_fb_connect_user_token');
      $fb_session = new FacebookSession($user_token);

      // Validate the session (token is not expired and token is from our app)
      $fb_session->validate();
      return $fb_session;
    }
    catch (\Exception $ex) {
      $this->loggerFactory->get('simple_fb_connect')->error('FB session validation failed. Exception: @message', array('@message' => ($ex->getMessage())));
    }
    return FALSE;
  }

  /**
   * Makes an API call to get user's Facebook profile
   *
   * @param \Facebook\FacebookSession $fb_session
   *   FacebookSession object
   *
   * @return \Facebook\GraphObject|false
   *   GraphObject representing the user
   *   False if exception was thrown
   */
  public function getFbProfile(FacebookSession $fb_session) {
    try {
      $request = new FacebookRequest($fb_session, 'GET', '/me?fields=id,name,email');
      $graphObject = $request->execute()->getGraphObject();
      return $graphObject;
    }
    catch (FacebookRequestException $ex) {
      $this->loggerFactory->get('simple_fb_connect')->error('Could not load FB user profile: FacebookRequestException. Error details: @message', array('@message' => json_encode($ex->getResponse())));
    }
    catch (\Exception $ex) {
      $this->loggerFactory->get('simple_fb_connect')->error('Could not load FB user profile: Unhandled exception. Error details: @message', array('@message' => ($ex->getMessage())));
    }

    // Something went wrong
    drupal_set_message(t('Error loading Facebook profile. Contact site administrator.'), 'error');
    return FALSE;
  }

  /**
   * Makes an API call to get the URL of user's Facebook profile picture
   *
   * @param Facebook\FacebookSession $fb_session
   *   FacebookSession object
   *
   * @return string|false
   *   Absolute URL of the profile picture
   *   False on errors
   */
  public function getFbProfilePicUrl(FacebookSession $fb_session) {
    // Determine preferred resolution for the profile picture.
    $resolution = $this->getPreferredResolution();

    // Generate FB API query.
    $query = '/me/picture?redirect=false';
    if (is_array($resolution)) {
      $query .= '&width=' . $resolution['width'] . '&height=' . $resolution['height'];
    }
    // Call Graph API to request profile picture
    try {
      $request = new FacebookRequest($fb_session, 'GET', $query);
      $url = $request->execute()->getGraphObject()->getProperty('url');
      return $url;
    }
    catch (FacebookRequestException $ex) {
      $this->loggerFactory->get('simple_fb_connect')->error('Could not load FB profile picture. FacebookRequestException: @message', array('@message' => json_encode($ex->getResponse())));
    }
    catch (\Exception $ex) {
      $this->loggerFactory->get('simple_fb_connect')->error('Could not load FB profile picture. Unhandled exception. Error details: @message', array('@message' => ($ex->getMessage())));
    }

    // Something went wrong and the picture could not be loaded
    return FALSE;
  }

  /**
   * Returns user's email address from Facebook profile
   *
   * @param Facebook\GraphObject $fb_profile
   *   User's Facebook profile
   *
   * @return string|false
   *   User's email address if found
   *   False otherwise
   */
  public function getEmail(GraphObject $fb_profile) {
    if ($email = $fb_profile->getProperty('email')) {
      return $email;
    }

    // Email address was not found. Log error and return FALSE
    $this->loggerFactory->get('simple_fb_connect')->error('No email address in FB user profile');
    return FALSE;
  }

  /**
   * Determines preferred profile pic resolution from account settings
   *
   * Return order: max resolution, min resolution, FALSE
   *
   * @return array|false
   *   Array of resolution, if defined in Drupal account settings
   *   False otherwise
   */
  protected function getPreferredResolution() {
    // Return order: max resolution, min resolution, FALSE
    if ($this->maxResolution) {
      $resolution = $this->maxResolution;
    }
    elseif ($this->minResolution) {
      $resolution = $this->minResolution;
    }
    else {
      return FALSE;
    }
    $dimensions = explode('x', $resolution);
    return array('width' => $dimensions[0], 'height' => $dimensions[1]);
  }

}
