<?php

/**
 * @file
 * Contains \Drupal\simple_fb_connect\SimpleFbConnectUserManager
 */
namespace Drupal\simple_fb_connect;

use Drupal\user\Entity\User;
use Drupal\Component\Utility\SafeMarkup;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\user\Plugin\views\argument_default\CurrentUser;
use Drupal\user\UserInterface;

/**
 * Contains all Simple FB Connect logic that is related to Drupal user
 * management.
 */
class SimpleFbConnectUserManager {

  protected $postLoginRoute;
  protected $registerBlockedByModule;
  protected $registerBlockedByDrupal;
  protected $loginBlockedForAdmin;
  protected $loginBlockedRoles;
  protected $pictureDirectory;
  protected $loggerFactory;
  protected $site_config;
  protected $user;


  /**
   * Constructor
   */
  public function __construct(ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory) {
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
    $module_config = $this->configFactory->get('simple_fb_connect.settings');
    $this->postLoginRoute = $module_config->get('post_login_route');
    $this->registerBlockedByModule = $module_config->get('login_only');
    $this->loginBlockedForAdmin = $module_config->get('disable_admin_login');
    $this->loginBlockedRoles = $module_config->get('disabled_roles');
    $this->user = $user;



    $user_config = $this->configFactory->get('user.settings');
    $this->registerBlockedByDrupal = 0;
    $this->newUserStatus = 0;
    if ($user_config->get('register') == 'admin_only') {
      $this->registerBlockedByDrupal = 1;
    }
    if ($user_config->get('register') == 'visitors') {
      $this->newUserStatus = 1;
    }

    $picture_config =  $this->configFactory->get('field.user.user.user_picture');
    $this->pictureDirectory = $picture_config->get('settings.file_directory');
  }

  /**
   * Returns Drupal user account by email
   *
   * @param string $email
   *   email address returned by Facebook
   *
   * @return \Drupal\user\Entity\User|false
   *   Drupal user account if found
   *   False otherwise
   */
  public function getUserByEmail($email) {
    return user_load_by_mail($email);
  }

  /**
   * Create a new user account
   *
   * @param string $name
   *   User's name on Facebook
   *
   * @param string $email
   *   User's email address
   *
   * @param string $fbid
   *   User's Facebook ID
   *
   * @param string $picture_url
   *   User's Facebook profile picture URL
   *
   * @todo Add other modules capability to modify user object
   *  https://www.drupal.org/node/2524336
   *
   * @todo Add Rules integration
   *  https://www.drupal.org/node/2524370
   *
   * @return \Drupal\user\Entity\User|false
   *   Drupal user account if user was created
   *   False otherwise
   */
  public function createUser($name, $email, $fbid, $picture_url) {
    // Make sure we have everyting we need
    if (!$name || !$email || !$fbid) {
      $this->loggerFactory->get(simple_fb_connect)->error('Failed to create user. Name: @name, email: @email, fbid: @fbid.', array(
        '@name' => $name,
        '@email' => $email,
        '@fbid' => $fbid
      ));
      drupal_set_message(t("Error while creating user account. Please contact site administrator."), 'error');
      return FALSE;
    }

    // Check if site configuration allow new users to register
    if ($this->registerBlockedByDrupal) {
      $this->loggerFactory->get(simple_fb_connect)->warning('Failed to create user. User registration is disabled in Drupal account settings.');
      drupal_set_message(t('Creation of new accounts on this site is disabled. Please contact site administrator.'), 'error');
      return FALSE;
    }

    // Check if module configuration allow new uses to register
    if ($this->registerBlockedByModule) {
      $this->loggerFactory->get(simple_fb_connect)->warning('Failed to create user. User registration is disabled in Simple FB Connect settings.');
      drupal_set_message(t('Only existing users can log in with Facebook.'), 'error');
      return FALSE;
    }

    // Drupal account's username will be user's name on Facebook.
    // Make sure that username will be unique.
    $username = $this->getUniqueUsername($name);

    // Password can be very long because user will never see this
    $password = user_password(32);

    //set up the user fields
    $fields = array(
      'name' => $username,
      'mail' => $email,
      'init' => $email,
      'pass' => $password,
      'status' => $this->newUserStatus,
    );

    // Try to download the profile picture and add it to user fields
    if (user_picture_enabled() && $picture_url) {
      if ($file = $this->downloadProfilePic($picture_url, $fbid)) {
        $fields['user_picture'] = $file->id();
      }
    }

    // Create new user account
    // TODO: Allow other modules to modify the user before it is saved
    // https://www.drupal.org/node/2524336
    $new_user = User::create($fields);
    $new_user->save();

    // Log notice and invoke Rules event if new user was succesfully created
    if ($new_user->id()) {
      $this->loggerFactory->get(simple_fb_connect)->notice('New user created. Username %username, UID: %uid', array(
        '%username' => $new_user->getUsername(),
        '%uid' => $new_user->id()
      ));

      // TODO: Add Rules integration here
      // https://www.drupal.org/node/2524370
      return $new_user;
    }

    // Something went wrong
    drupal_set_message(t('Creation of user account failed. Please contact site administrator.'), 'error');
    $this->loggerFactory->get(simple_fb_connect)->error('Could not create new user.');
    return FALSE;
  }

  /**
   * Logs the user in.
   *
   * @todo Add Rules integration
   *   https://www.drupal.org/node/2524370
   *
   * @todo Add Boost integtraion when Boost is available for D8
   *   https://www.drupal.org/node/2524372
   *
   * @param Drupal\user\Entity\User $drupal_user
   *
   * @return bool
   *   True if login was successful
   *   False if the login was blocked
   */
  public function loginUser(User $drupal_user) {
    // Prevent admin login if defined in module settings
    if ($this->loginDisabledForAdmin($drupal_user)) {
      drupal_set_message(t('FB login is disabled for site administrator. Login with your local user account.'), 'error');
      return FALSE;
    }

    // Prevent login if user has one of the roles defined in module settings
    if ($this->loginDisabledByRole($drupal_user)) {
      drupal_set_message(t('FB login is disabled for your role. Please login with your local user account.'), 'error');
      return FALSE;
    }

    // Check that the account is active and log the user in
    if ($drupal_user->isActive()) {
      user_login_finalize($drupal_user);

      // TODO: Invoke a login event if Rules module is enabled
      // https://www.drupal.org/node/2524370

      // TODO: Add Boost cookie if Boost module is enabled
      // https://www.drupal.org/node/2524372

      drupal_set_message(t('You are now logged in as %username.', array('%username' => $drupal_user->getUsername())));
      return TRUE;
    }

    // If we are still here, account is blocked.
    drupal_set_message(t('You could not be logged in because your user account %username is not active.', array('%username' => $drupal_user->getUsername())), 'warning');
    $this->loggerFactory->get(simple_fb_connect)->warning('FB login for user %user prevented. Account is blocked.', array('%user' => $drupal_user->getUsername()));
    return FALSE;
  }

  /**
   * Returns the route the user should be redirected after a successful login
   *
   * @todo Implement destination handling
   *   https://www.drupal.org/node/2524328
   *
   * @return string
   *   Druapl route where the user should be redirected after FB login
   */
  public function getPostLoginRoute() {
    // TODO: If we have ?destination in Drupal session, use that instead
    // https://www.drupal.org/node/2524328
    return $this->postLoginRoute;
  }

  /**
   * Ensures that Drupal username will be unique
   *
   * @param string $fb_name
   */
  protected function getUniqueUsername($fb_name) {
    $base = trim($fb_name);
    $i = 1;
    $candidate = $base;
    while (is_object(user_load_by_name($candidate))) {
      $i++;
      $candidate = $base . " " . $i;
    }
    return $candidate;
  }

  /**
   * Downloads the profile picture to Drupal filesystem
   *
   * @param string $picture_url
   *   Absolute URL where to download the profile picture
   *
   * @param string $fbid
   *   Facebook ID of the user
   *
   * @return \Drupal\file\FileInterface|false
   *   FileInterface object if file was succesfully downloaded
   *   False otherwise
   */
  protected function downloadProfilePic($picture_url, $fbid) {
    // Make sure that we have everything we need
    if (!$picture_url || !$fbid) {
      return FALSE;
    }

    // Check target directory from account settings and make sure it's writeable
    $directory = file_default_scheme() . '://' . $this->pictureDirectory;
    if (!file_prepare_directory($directory, FILE_CREATE_DIRECTORY)) {
      $this->loggerFactory->get(simple_fb_connect)->error('Could not save FB profile picture. Directory is not writeable: @directory', array('@dir' => $directory));
    }

    // Download the picture. Facebook API always serves the images in jpg format.
    $destination = $directory . '/' . SafeMarkup::checkPlain($fbid) . '.jpg';
    if ($file = system_retrieve_file($picture_url, $destination, TRUE, FILE_EXISTS_REPLACE)) {
      return $file;
    }

    // Something went wrong
    $this->loggerFactory->get(simple_fb_connect)->error('Could not save FB profile picture. Unhandled error.');
    return FALSE;
  }

  /**
   * Checks if the admin FB login is disabled
   *
   * @param \Drupal\user\Entity\User $drupal_user
   *
   * @return bool
   *   True if login is disabled for this user
   *   False if login is not disabled for this user
   */
  protected function loginDisabledForAdmin(User $drupal_user) {
    if ($drupal_user->id() == 1 && $this->loginBlockedForAdmin) {
      $this->loggerFactory->get(simple_fb_connect)->warning('FB login for user %user prevented. FB login for site administrator (user 1) is disabled in module settings.', array('%user' => $drupal_user->getUsername()));
      return TRUE;
    }

    // User is not admin or admin login is not disabled
    return FALSE;
  }

  /**
   * Checks if the user has one of the "FB login disabled" roles
   *
   * @param \Drupal\user\Entity\User $drupal_user
   *
   * @return bool
   *   True if login is disabled for one of this user's role
   *   False if login is not disabled for this user's roles
   */
  protected function loginDisabledByRole(User $drupal_user) {
    // Loop through all roles the user has
    foreach ($drupal_user->getRoles() as $role) {
      // Check if FB login is disabled for this role. Disabled roles have value > 0
      if (array_key_exists($role, $this->loginBlockedRoles) && ($this->loginBlockedRoles[$role] !== 0)) {
        $this->loggerFactory->get(simple_fb_connect)->warning('FB login for user %user prevented. FB login for role %role is disabled in module settings.', array(
          '%user' => $drupal_user->getUsername(),
          '%role' => $role
        ));
        return TRUE;
      }
    }

    // If we didn't return TRUE already, FB login is not disabled for any of the user's roles
    return FALSE;
  }

}
